import * as ts from "typescript";
import { ApiItemOptions } from "./abstractions/api-item";
import { ApiType, ApiTypeKind, ApiBaseType, ApiBasicType, ApiReferenceBaseType, ApiReferenceType, ApiUnionOrIntersectionType, IndexedAccessType, ArrayType, ParenthesizedType, TupleType, TypeOperatorType, TypePredicateType, TypeQueryType } from "./contracts/api-type";
export declare namespace ApiTypeHelpers {
    function ResolveApiType(options: ApiItemOptions, type: ts.Type, typeNode?: ts.TypeNode, self?: boolean): ApiType;
    /**
     * @internal
     */
    function typeNodeToBaseType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeNode): ApiBaseType;
    /**
     * @internal
     */
    interface ResolvedTypeItemReference {
        name?: string;
        referenceId?: string;
    }
    /**
     * @internal
     */
    function resolveTypeItemReference(symbol: ts.Symbol, options: ApiItemOptions): ResolvedTypeItemReference;
    function ResolveApiBasicType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeNode): ApiBasicType;
    /**
     * Resolving `Definition` reference.
     * @param kind `ApiReferenceBaseType` ApiTypeKind value
     */
    function ReferenceBaseTypeToTypeDto(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeNode, kind?: ApiTypeKind): ApiReferenceBaseType;
    /**
     * Example `Foo`.
     */
    function TypeReferenceNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeReferenceType, self?: boolean): ApiReferenceType;
    /**
     * Example `string | number` or `string & number`.
     */
    function TypeNodeUnionOrIntersectionToApiType(options: ApiItemOptions, type: ts.UnionOrIntersectionType, typeNode: ts.UnionTypeNode | ts.IntersectionTypeNode): ApiUnionOrIntersectionType;
    /**
     * Example `string[]`.
     */
    function ArrayTypeNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.ArrayTypeNode): ArrayType;
    /**
     * Example `[string, number]`.
     */
    function TupleTypeNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TupleTypeNode): TupleType;
    /**
     * Example `arg is string`.
     */
    function TypePredicateNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypePredicateNode): TypePredicateType;
    /**
     * Example `keyof Foo`.
     */
    function TypeOperatorNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeOperatorNode): TypeOperatorType;
    /**
     * Example: `Foo[T]`.
     */
    function IndexedAccessTypeNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.IndexedAccessTypeNode): IndexedAccessType;
    /**
     * Example: `(string | number)`.
     */
    function ParenthesizedTypeNodeToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.ParenthesizedTypeNode): ParenthesizedType;
    /**
     * Example: `typeof Foo`.
     */
    function TypeQueryToApiType(options: ApiItemOptions, type: ts.Type, typeNode: ts.TypeQueryNode): TypeQueryType;
    type HeritageKinds = ts.SyntaxKind.ImplementsKeyword | ts.SyntaxKind.ExtendsKeyword;
    function GetHeritageList(heritageClauses: ts.NodeArray<ts.HeritageClause>, kind: HeritageKinds, options: ApiItemOptions): ApiType[];
}
